/* 
*  Simple robot control with Arduino & the CC3000 WiFi chip
*/

// Include required libraries
#include <Adafruit_CC3000.h>
#include <ccspi.h>
#include <SPI.h>
#include <string.h>
#include "utility/debug.h"
#include <stdlib.h>

String result;
int motorCommand[5];
// will be used to parse values from the json file on the server:  {"mode":"1","xval":"0","yval":"0","udlr":"","cmdVal":""}
int resultLength;

//Nano and UNO PWM pins:  3,5,6,9,10,11
#define leftMtrSpdPin 6  //maps to pin 0 on Digispark Motor Shield - needs to be a PWM pin
#define rightMtrSpdPin 9 //maps to pin 1 on Digispark Motor Shield - needs to be a PWM pin
#define leftMtrDirPin 7  //maps to pin 2 on Digispark Motor Shield
#define rightMtrDirPin 4 //maps to pin 5 on Digispark Motor Shield
#define LEDStatPin 2
// Define CC3000 chip pins
#define ADAFRUIT_CC3000_IRQ   3
#define ADAFRUIT_CC3000_VBAT  5
#define ADAFRUIT_CC3000_CS    10

 
const unsigned long
  dhcpTimeout     = 60L * 1000L, // Max time to wait for address from DHCP
  connectTimeout  = 15L * 1000L, // Max time to wait for server connection
  responseTimeout = 15L * 1000L; // Max time to wait for data from server
uint32_t t;


// WiFi network (change with your settings !)
#define WLAN_SSID       "[YOURSSIDNAMEHERE]"        // cannot be longer than 32 characters!
#define WLAN_PASS       "[YOUR NETWORK PASSWORD HERE]"
#define WLAN_SECURITY   WLAN_SEC_WPA2 // This can be WLAN_SEC_UNSEC, WLAN_SEC_WEP, WLAN_SEC_WPA or WLAN_SEC_WPA2

// Create CC3000 instances
Adafruit_CC3000 cc3000 = Adafruit_CC3000(ADAFRUIT_CC3000_CS, ADAFRUIT_CC3000_IRQ, ADAFRUIT_CC3000_VBAT,
                                         SPI_CLOCK_DIV2);                                
                                         
// Local server IP, port, and repository (change with your settings !)
//uint32_t ip = cc3000.IP2U32(192,168,0,1);
uint32_t ip = cc3000.IP2U32(192,168,2,10);
int port = 80;
//String repository = "/arduino-robot-nn/";
//path off the root of the site
String repository = "/CC3000Sample/";


void setup() {
   
  Serial.begin(115200);
  
  result = "";
  
  pinMode(leftMtrSpdPin, OUTPUT);   
  pinMode(leftMtrDirPin, OUTPUT);
  pinMode(rightMtrDirPin, OUTPUT);
  pinMode(rightMtrSpdPin, OUTPUT);    
  pinMode(LEDStatPin, OUTPUT);    
 
 
  // Initialise the CC3000 module
  if (!cc3000.begin())
  {
    while(1);  //wait while it initializes
  }

  // Connect to  WiFi network
  cc3000.connectToAP(WLAN_SSID, WLAN_PASS, WLAN_SECURITY);
  Serial.println("Connected to WiFi network!");
    
  // Check DHCP
  Serial.println(F("Requesting address from DHCP server..."));
  for(t=millis(); !cc3000.checkDHCP() && ((millis() - t) < dhcpTimeout); delay(1000));
  if(cc3000.checkDHCP()) {
    Serial.println(F("OK"));
  } else {
    Serial.println(F("failed"));
    return;
  }
 
}

//##########################################################################################################################

void loop() {
  
  // Send request to get relay state
  Adafruit_CC3000_Client www = cc3000.connectTCP(ip, port);
  if (www.connected()) {
    Serial.println(F("Connected !"));   
    www.println("GET " + repository + "server.php HTTP/1.0");  //this calls on server.php to read the contents of the json file and echo them back for processing
    www.fastrprintln(F("Connection: close"));
    www.fastrprintln(F(""));
  } else {
    Serial.println(F("Connection failed"));    
    return;
  }
  
  Serial.println("got here 1");
  
  while (www.connected()) {
    while (www.available()) {
      char c = www.read();
      result = result + c;
      
      // Delete HTTP headers
      //if(result.endsWith("Content-Type: text/html"))
      result.toLowerCase();
      if(result.endsWith("content-type: text/html"))
      {
        //if true, it will essentially clear out all the header content that came before
        //the actual motor values x,x,x,x,  later this is also trimmed as there are spaces around it
        result="";
      }
    }
  }
  www.close();
  
  //Serial.println(result);

  
 // Format result and extract the variables
 format_result(motorCommand,result);
 Serial.println("got here 2");
 
 //blink LED
 digitalWrite(LEDStatPin, HIGH);
 delay(50);
 digitalWrite(LEDStatPin, LOW);
 delay(50);
  
 // Print received values
 //Serial.println("Motor 1 speed: " + String(motorCommand[0]) + " and direction: " + String(motorCommand[2]));
 //Serial.println("Motor 2 speed: " + String(motorCommand[1]) + " and direction: " + String(motorCommand[3]));
 Serial.println("Mode: " + String(motorCommand[0]));
 Serial.println("Motor 1 speed/dir: " + String(motorCommand[1]));
 Serial.println("Motor 2 speed/dir: " + String(motorCommand[2]));
 Serial.println("UDLRVal: " + String(motorCommand[3]));
 Serial.println("CommandVal: " + String(motorCommand[4]));
 // Send motor commands
// send_motor_command(speed_motor1,direction_motor1,motorCommand[0],motorCommand[2]);
// send_motor_command(speed_motor2,direction_motor2,motorCommand[1],motorCommand[3]);
 //send_motor_command(leftMtrSpdPin,leftMtrDirPin,motorCommand[0]);
 //send_motor_command(rightMtrSpdPin,rightMtrDirPin,motorCommand[1]);
 Serial.println("got here 3");
 //flip through the mode values and react accordingly
 switch (motorCommand[0]) {
    case 1:
      //driving via joystick x,y values
      if (motorCommand[1] != 0 && motorCommand[2] != 0)
        {
          Serial.println("got here 4");
          driveMotors(motorCommand[1],motorCommand[2]);
        } 
        else
        {
         Serial.println("got here 5"); 
         analogWrite(leftMtrSpdPin, 0);   //drive the motor
         analogWrite(rightMtrSpdPin, 0);  //drive the motor
        }
      break;
    case 2:
      //driving by simple joystick U D L R
      //write code here to move the robot based on these values.
      break;
    case 3:
      //sending command values
      //write your own code here to react to command values 1 - 8
      break;  
    default:
         Serial.println("got here 6"); 
         analogWrite(leftMtrSpdPin, 0);   //drive the motor
         analogWrite(rightMtrSpdPin, 0);  //drive the motor
     break;
  }




 
 // Reset result variable
 result = "";      
}
//##########################################################################################################################

//parse the content and assign the mode, motor and command values
void format_result(int* array, String result) {
 
 result.trim();
 
 //remove the last comma
 //if (result.endsWith(","))
 //{
 //result = result.substring(0, result.length()-1);
 //}
 // result = result.substring(0, result.lastIndexOf(','));
 resultLength = result.length();
 //Serial.println(result);
 //Serial.print(" result length = ");
 //Serial.println(resultLength);
 
 int commaPosition;
 int i = 0;
 do
  {
      commaPosition = result.indexOf(',');
      
      if(commaPosition != -1)
      {
          //Serial.println( result.substring(0,commaPosition));
          array[i] = result.substring(0,commaPosition).toInt();

          //Serial.print("CommaPos: " );
          //Serial.print (commaPosition);
          //Serial.print("  MotorVals: " );
          //Serial.print(array[i]);
          i = i+1;
          result = result.substring(commaPosition+1, result.length());
          //Serial.print("  NextResult: " );
          //Serial.println(result);         
      }
      else
      {
         if(result.length() > 0) {
           Serial.println(result);
          }
      }     
   }
   while(commaPosition >=0);  
} 



  

void driveMotors(int xVal, int yVal)
{ 
  //both motors forward full speed
  //digitalWrite(MotorADir, HIGH);   //forward
  //digitalWrite(MotorBDir, LOW); 
  //analogWrite(MotorASpeed, 100); //100 will be the lowest speed to start at.
  //analogWrite(MotorBSpeed, 100);
  
  float xPct=1.0;
  int xAdj, yAdj;
  //xAdj = map(abs(xVal), 0, 100, 50, 255);  //start motor speed of 50 is as low as these motors will work.  Your motors may be different
  //yAdj = map(abs(yVal), 0, 100, 50, 255);  //start motor speed of 50 is as low as these motors will work.  Your motors may be different   
   
   //xAdj = map(abs(xVal), 0, 100, baseVal, 255);
   //yAdj = map(abs(yVal), 0, 100, baseVal, 255);
Serial.println("got here 4a");
   xAdj = map(abs(xVal), 0, 100, 50, 255);
   yAdj = map(abs(yVal), 0, 100, 50, 255);
Serial.println("got here 4b");

  //xPct = map(abs(xAdj), 0, 255, 1, 0);  
  xPct = mapf((float)abs(xAdj), (float)0, (float)255, (float)1, (float)0);  
Serial.println("got here 4c");  
 //Serial.print("  xadj: "); printDouble(xAdj,10);
 //Serial.print("  yadj: "); printDouble(yAdj,10);
 //Serial.print("  yVal: "); printDouble(yVal,10);
 //Serial.print("  xAdj: "); printDouble(xAdj,10);
 //Serial.print("  xVal: "); printDouble(xVal,10);
 //Serial.print("  x%: "); printDouble(xPct,10);   
 //drive based on quadrant

  
  if (((xVal > -20) && (xVal < 20)) && (((yVal <= 100) && (yVal > 60)) || ((yVal >= -100) && (yVal < -60))))
  {
    Serial.println("got here 4d");
    if  ((yVal <= 100) && (yVal > 60))    //1:  joystick full fwd.  Both motors full speed ahead.
    {
      Serial.println("got here 4e");
      digitalWrite(leftMtrDirPin, HIGH);  
      digitalWrite(rightMtrDirPin, LOW); //yes - messed up soldering motor leads - LOW makes FWD in this case.
      analogWrite(leftMtrSpdPin, 255);   //drive the motor
      analogWrite(rightMtrSpdPin, 255);  //drive the motor
      //Serial.print(" - Full FWD");   
      Serial.println("got here 4f");
    }
    else if ((yVal >= -100) && (yVal < -60))  //2:  joystick full rev. Both motors full speed rev.
    {
      digitalWrite(leftMtrDirPin, LOW);  
      digitalWrite(rightMtrDirPin, HIGH); //yes - messed up soldering motor leads - HIGH makes REV in this case.
      analogWrite(leftMtrSpdPin, 255);   //drive the motor
      analogWrite(rightMtrSpdPin, 255);  //drive the motor     
      //Serial.print(" - Full REV");   
    }
    
  }
  else if  (((yVal > -20) && (yVal < 20)) && (((xVal <= 100) && (xVal > 60)) || ((xVal >= -100) && (xVal < -60))))
  {
    if ((xVal <= 100) && (xVal > 60))  //3:  Fast turn right. Rt Mtr full fwd. Left Mtr full rev.
    {
      digitalWrite(leftMtrDirPin, HIGH);  
      digitalWrite(rightMtrDirPin, HIGH); 
      analogWrite(leftMtrSpdPin, 150);   //drive the motor
      analogWrite(rightMtrSpdPin, 150);  //drive the motor
      //Serial.print(" - Full RT");   
    }
    else if ((xVal >= -100) && (xVal < -60))  //4: Fast turn left: Left Mtr full fwd. Rt Mtr full rev.
    {
      digitalWrite(leftMtrDirPin, LOW);  
      digitalWrite(rightMtrDirPin, LOW); 
      analogWrite(leftMtrSpdPin, 150);   //drive the motor
      analogWrite(rightMtrSpdPin, 150);  //drive the motor
      //Serial.print(" - Full LF");   
    }
  }
   
  // if not extreme corners, do normal movements. 
  else if (yVal > 0)  //going forward
  {
    //robot driving fwd - now determine left and right.
    digitalWrite(leftMtrDirPin, HIGH);  
    digitalWrite(rightMtrDirPin, LOW); 
    
    if (xVal < -15) //going left  - leave room for going straight
    {
      analogWrite(leftMtrSpdPin, yAdj * xPct);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj);  //drive the motor
      //Serial.print("FwLF: "); printDouble(xPct,10);
    }
    else if (xVal > 15)
    {  //going right
      analogWrite(leftMtrSpdPin, yAdj);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj  * xPct);  //drive the motor    
      //Serial.print("FwRT: "); printDouble(xPct,10);
    }
    else  //x is beteen -15 and 15 - assume going straight
    {
      analogWrite(leftMtrSpdPin, yAdj);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj);   //drive the motor    
      //Serial.print("Normal FWD: ");
    }
  }
  else   //y val is < 0 - reversing
  {
    //robot driving backwards - now determine left and right.
    digitalWrite(leftMtrDirPin, LOW);  
    digitalWrite(rightMtrDirPin, HIGH); 

    if (xVal < -15) //going left  - leave room for going straight
    {
      analogWrite(leftMtrSpdPin, yAdj * xPct);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj);  //drive the motor
      //Serial.print("RevLF: "); printDouble(xPct,10);
    }
    else if (xVal > 15)
    {
      analogWrite(leftMtrSpdPin, yAdj);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj  * xPct);  //drive the motor    
      //Serial.print("RevRT: "); printDouble(xPct,10);
    }
    else  //x is beteen -15 and 15 - assume going straight
    {
      analogWrite(leftMtrSpdPin, yAdj);    //drive the motor
      analogWrite(rightMtrSpdPin, yAdj);   //drive the motor    
      //Serial.print("NormalRev: "); 
    }

  }
  
  //set the motors to off
  //analogWrite(leftMtrSpdPin, 0);    //drive the motor
  //analogWrite(rightMtrSpdPin, 0);   //drive the motor   
   //slight delay
  delay(50);
  //Serial.println();
  //leftSpeed=0, rightSpeed=0, xAdj=0, yAdj=0;
Serial.println("got here 4 done");
}


//void printDouble( double val, unsigned int precision){
// prints val with number of decimal places determine by precision
// NOTE: precision is 1 followed by the number of zeros for the desired number of decimial places
// example: printDouble( 3.1415, 100); // prints 3.14 (two decimal places)

//    Serial.print (int(val));  //prints the int part
//    Serial.print("."); // print the decimal point
//    unsigned int frac;
//    if(val >= 0)
//        frac = (val - int(val)) * precision;
//    else
//        frac = (int(val)- val ) * precision;
//    Serial.println(frac,DEC);
//}   
 
//use to map values to float.   
float mapf (float x, float in_min, float in_max, float out_min, float out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}


//Depreciated
void send_motor_command(int speed_pin, int direction_pin, int pwm)
{
  analogWrite(speed_pin, pwm);
  
  //analogWrite(speed_pin,pwm); // Set PWM control, 0 for stop, and 255 for maximum speed
}

